﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Threading.Tasks;

namespace WpfApp1.UDP
{

    #region 델리게이트
    public delegate void DataUpdateEventHandler(int dataSize);
    public delegate void RecvUpdateEventHandler(int dataSize);
    public delegate void SendErrorMessageOutput(string msg);
    #endregion

    class UdpProtocol
    {
        #region 전역변수
        private const int buffSize = 65535;
        private StateObject state = new StateObject();
        private CancellationTokenSource source = new CancellationTokenSource();
        private Socket socket;
        private EndPoint ep = new IPEndPoint(IPAddress.Any, 0);
        private AsyncCallback recv = null;
        public IPEndPoint RemoteEndPoint;
        Task send_task;
        private bool b_heartBeatThread;
        #endregion

        #region 델리게이트
        public DataUpdateEventHandler DataSendEvent;
        public RecvUpdateEventHandler DataRecvEvent;
        public SendErrorMessageOutput ErrorMsg;
        #endregion

        private class StateObject
        {
            public byte[] buffer = new byte[buffSize];
        }

        public UdpProtocol()
        {

        }

        #region connect
        public int Connect(string ipNumber, int portNumber)
        {
            // ip, port 설정.
            bool connect = false;
            IPAddress ipAddress;
            if (IPAddress.TryParse(ipNumber, out ipAddress))
            {
                RemoteEndPoint = new IPEndPoint(ipAddress, portNumber);
                socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
                socket.Connect(RemoteEndPoint);
                connect = true;
                b_heartBeatThread = true;
            }
            else
            {
                foreach (IPAddress ip in Dns.GetHostEntry(ipNumber).AddressList)
                {
                    try
                    {
                        RemoteEndPoint = new IPEndPoint(ip, portNumber);
                        socket.Connect(RemoteEndPoint);
                        connect = true;
                        break;
                    }
                    catch (Exception) { }
                }
            }
            if (!connect)
            {
                throw new Exception("Unable to connect");
            }

            Receive();

            //send_task = new Task(new Action<object>(HeartBeatThread), socket, source.Token);

            send_task = new Task(new Action<object>(HeartBeatThread), socket, source.Token);

            send_task.Start();

            return 0;
        }
        #endregion

        #region Send, SendThread
        public int SendFun(string data, int data_len)
        {
            // 보낼 데이터 세팅
            byte[] x = Encoding.UTF8.GetBytes(data);
            Send(RemoteEndPoint, x);
            return 0;
        }

        public void HeartBeatThread(object socket)
        {
            if (socket == null)
            {
                ErrorMsg("HeartBeatThread Function Socket Error");
            }
            Thread.Sleep(1000);
            string startData = "START";
            int dataSize = startData.Length;
            while (b_heartBeatThread)
            {
                SendFun(startData, dataSize);
                DataSendEvent(dataSize); // 델리게이트
                Thread.Sleep(900);
            }
        }

        internal void Send(IPEndPoint endPoint, byte[] data)
        {
            PacketSending(endPoint, data, data.Length);
            socket.BeginSendTo(data, 0, data.Length, SocketFlags.None, endPoint, (ar) =>
            {
                try
                {
                    StateObject so = (StateObject)ar.AsyncState;
                    int bytes = socket.EndSend(ar);
                }
                catch (Exception ex)
                {
                    var x = ex;
                }
            }, state);
        }
        #endregion

        #region DisConnect
        public int DisConnect()
        {
            if (socket == null)
            {
                return 1;
            }
            else
            {
                b_heartBeatThread = false;
                send_task.Wait();
                send_task = null;
                source.Cancel();
                socket.Close();
                //socket = null;
                return 0;
            }
        }
        #endregion


        private void Receive()
        {
            socket.BeginReceiveFrom(state.buffer, 0, buffSize, SocketFlags.None, ref ep, recv = (ar) =>
            {
                StateObject so = (StateObject)ar.AsyncState;
                try
                {
                    int bytes = socket.EndReceiveFrom(ar, ref ep);
                    byte[] data = new byte[bytes];
                    Buffer.BlockCopy(so.buffer, 0, data, 0, bytes);
                    socket.BeginReceiveFrom(so.buffer, 0, buffSize, SocketFlags.None, ref ep, recv, so);
                    PacketReceive((IPEndPoint)ep, data, bytes);
                }
                catch (Exception ex)
                {
                    var x = ex;
                }
            }, state);
        }

        public virtual int PacketSending(IPEndPoint endPoint, byte[] data, int length)
        {
            return -1;
        }

        public virtual void PacketReceive(IPEndPoint ep, byte[] data, int length)
        {
            DataRecvEvent(length);
        }
    }
}
