﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WpfApp1.xaml
{
    /// <summary>
    /// UdpPage.xaml에 대한 상호 작용 논리
    /// </summary>
    public partial class UdpPage : Page
    {
        #region 전역변수
        private UDP.UdpProtocol udpProtocol;
        string ipNumber;
        int portNumber;
        #endregion

        public UdpPage()
        {
            // TITLE
            string subTitle = "test";

            // IP
            ipNumber = "192.168.0.2";
            IPAddress ip = IPAddress.Parse(ipNumber);
            byte[] ipbytes = ip.GetAddressBytes();

            //PORT
            portNumber = 40001;

            //=====================
            InitializeComponent();
            //=====================

            Title.Content = subTitle;
            IPBox.Text = ipNumber;
            PortBox.Text = portNumber.ToString();
        }

        private void Page_Initialized(object sender, EventArgs e)
        {
            udpProtocol = new UDP.UdpProtocol();

            // 생성 될 때 최초 Connect
            if (udpProtocol.Connect(ipNumber, portNumber) == 0)
            {
                udpProtocol.DataSendEvent += new UDP.DataUpdateEventHandler(this.HeartBeatThread); // 델리게이트
                udpProtocol.ErrorMsg += new UDP.SendErrorMessageOutput(this.SendErrorMessageOutput);
                udpProtocol.DataRecvEvent += new UDP.RecvUpdateEventHandler(this.RecvThreadValue);
                ColorBox.Fill = Brushes.Lime; // 색 변화
            }
            else
            {
                ColorBox.Fill = Brushes.Red;
            }
        }

        private void Connect_Click(object sender, RoutedEventArgs e)
        {
            if(udpProtocol.Connect(ipNumber, portNumber) == 0)
            {
                infoBox.AppendText("Connect!!" + Environment.NewLine);
                infoBox.ScrollToEnd();
                ColorBox.Fill = Brushes.Lime; // 색 변화
            }
        }

        private void DisConnect_Click(object sender, RoutedEventArgs e)
        {
            if(udpProtocol.DisConnect() == 0)
            {
                infoBox.AppendText("DisConnect" + Environment.NewLine);
                infoBox.ScrollToEnd();
            }
            else
            {
                infoBox.AppendText("DisConnect Error" + Environment.NewLine);
                infoBox.ScrollToEnd();
            }
        }

        #region Recv 정보 출력 함수
        public void RecvThreadValue(int dataSize)
        {
            recvBox.Dispatcher.BeginInvoke(new Action<int>(UdpRecvDataPrint), dataSize);
        }

        private void UdpRecvDataPrint(int msg)
        {
            recvBox.AppendText("[RECV] " + msg + " Bytes" + Environment.NewLine);
            recvBox.ScrollToEnd();
        }
        #endregion

        #region HeartBeatThread 정보 출력 함수
        public void HeartBeatThread(int dataSize)
        {
            infoBox.Dispatcher.BeginInvoke(new Action<int>(HeartBeatThreadOuput), dataSize);
        }

        private void HeartBeatThreadOuput(int dataLen)
        {
            infoBox.AppendText("[SEND] " + dataLen + " Bytes" + Environment.NewLine);
            infoBox.ScrollToEnd();
        }
        #endregion

        #region Error Message output 함수
        public void SendErrorMessageOutput(string msg)
        {
            infoBox.AppendText("[ERROR] " + msg + Environment.NewLine);
            infoBox.ScrollToEnd();
        }
        #endregion
    }
}
